@@ -88,47 +88,42 @@ MediaObject::MediaObject(Backend *backend, QObject *parent)
     static int count = 0;
     m_name = "MediaObject" + QString::number(count++);
 
-    if (!m_backend->isValid()) {
-        setError(tr("Cannot start playback. \n\nCheck your GStreamer installation and make sure you "
-                    "\nhave libgstreamer-plugins-base installed."), Phonon::FatalError);
-    } else {
-        m_root = this;
-        m_pipeline = new Pipeline(this);
-        m_isValid = true;
-        GlobalSubtitles::instance()->register_(this);
+    m_root = this;
+    m_pipeline = new Pipeline(this);
+    m_isValid = true;
+    GlobalSubtitles::instance()->register_(this);
 
-        connect(m_pipeline, SIGNAL(aboutToFinish()),
-                this, SLOT(handleAboutToFinish()), Qt::DirectConnection);
-        connect(m_pipeline, SIGNAL(eos()),
-                this, SLOT(handleEndOfStream()));
-        connect(m_pipeline, SIGNAL(warning(QString)),
-                this, SLOT(logWarning(QString)));
-        connect(m_pipeline, SIGNAL(durationChanged(qint64)),
-                this, SLOT(handleDurationChange(qint64)));
-        connect(m_pipeline, SIGNAL(buffering(int)),
-                this, SIGNAL(bufferStatus(int)));
-        connect(m_pipeline, SIGNAL(stateChanged(GstState,GstState)),
-                this, SLOT(handleStateChange(GstState,GstState)));
-        connect(m_pipeline, SIGNAL(errorMessage(QString,Phonon::ErrorType)),
-                this, SLOT(setError(QString,Phonon::ErrorType)));
-        connect(m_pipeline, SIGNAL(metaDataChanged(QMultiMap<QString,QString>)),
-                this, SIGNAL(metaDataChanged(QMultiMap<QString,QString>)));
-        connect(m_pipeline, SIGNAL(availableMenusChanged(QList<MediaController::NavigationMenu>)),
-                this, SIGNAL(availableMenusChanged(QList<MediaController::NavigationMenu>)));
-        connect(m_pipeline, SIGNAL(videoAvailabilityChanged(bool)),
-                this, SIGNAL(hasVideoChanged(bool)));
-        connect(m_pipeline, SIGNAL(seekableChanged(bool)),
-                this, SIGNAL(seekableChanged(bool)));
-        connect(m_pipeline, SIGNAL(streamChanged()),
-                this, SLOT(handleStreamChange()));
+    connect(m_pipeline, SIGNAL(aboutToFinish()),
+            this, SLOT(handleAboutToFinish()), Qt::DirectConnection);
+    connect(m_pipeline, SIGNAL(eos()),
+            this, SLOT(handleEndOfStream()));
+    connect(m_pipeline, SIGNAL(warning(QString)),
+            this, SLOT(logWarning(QString)));
+    connect(m_pipeline, SIGNAL(durationChanged(qint64)),
+            this, SLOT(handleDurationChange(qint64)));
+    connect(m_pipeline, SIGNAL(buffering(int)),
+            this, SIGNAL(bufferStatus(int)));
+    connect(m_pipeline, SIGNAL(stateChanged(GstState,GstState)),
+            this, SLOT(handleStateChange(GstState,GstState)));
+    connect(m_pipeline, SIGNAL(errorMessage(QString,Phonon::ErrorType)),
+            this, SLOT(setError(QString,Phonon::ErrorType)));
+    connect(m_pipeline, SIGNAL(metaDataChanged(QMultiMap<QString,QString>)),
+            this, SIGNAL(metaDataChanged(QMultiMap<QString,QString>)));
+    connect(m_pipeline, SIGNAL(availableMenusChanged(QList<MediaController::NavigationMenu>)),
+            this, SIGNAL(availableMenusChanged(QList<MediaController::NavigationMenu>)));
+    connect(m_pipeline, SIGNAL(videoAvailabilityChanged(bool)),
+            this, SIGNAL(hasVideoChanged(bool)));
+    connect(m_pipeline, SIGNAL(seekableChanged(bool)),
+            this, SIGNAL(seekableChanged(bool)));
+    connect(m_pipeline, SIGNAL(streamChanged()),
+            this, SLOT(handleStreamChange()));
 
-        connect(m_pipeline, SIGNAL(textTagChanged(int)),
-                this, SLOT(getSubtitleInfo(int)));
-        connect(m_pipeline, SIGNAL(trackCountChanged(int)),
-                this, SLOT(handleTrackCountChange(int)));
+    connect(m_pipeline, SIGNAL(textTagChanged(int)),
+            this, SLOT(getSubtitleInfo(int)));
+    connect(m_pipeline, SIGNAL(trackCountChanged(int)),
+            this, SLOT(handleTrackCountChange(int)));
 
-        connect(m_tickTimer, SIGNAL(timeout()), SLOT(emitTick()));
-    }
+    connect(m_tickTimer, SIGNAL(timeout()), SLOT(emitTick()));
 }
 
 MediaObject::~MediaObject()
@@ -378,9 +373,6 @@ qint64 MediaObject::getPipelinePos() const
  */
 void MediaObject::setSource(const MediaSource &source)
 {
-    if (!isValid())
-        return;
-
     DEBUG_BLOCK;
 
     if (source.type() == Phonon::MediaSource::Invalid) {
@@ -453,9 +445,6 @@ void MediaObject::stop()
 
 void MediaObject::seek(qint64 time)
 {
-    if (!isValid())
-        return;
-
     DEBUG_BLOCK;
 
     if (m_waitingForNextSource) {
@@ -861,23 +850,42 @@ void MediaObject::handleAboutToFinish()
     m_aboutToFinishLock.lock();
     m_handlingAboutToFinish = true;
     emit aboutToFinish();
-    // Three seconds should be more than enough for any application to get their act together.
-    // Any longer than that and they have bigger issues.  If Phonon does no supply a next source
-    // within 3 seconds, treat as if there is no next source to come, and finish the current source.
+    // As our signal gets emitted queued we need to wait here until either a
+    // new source or a timeout is reached.
+    // If we got a new source in time -> hooray + gapless
+    // If we did not get a new source in time -> boooh + stop()
     if (!m_skipGapless) {
-      if (m_aboutToFinishWait.wait(&m_aboutToFinishLock, 3000)) {
-          debug() << "Finally got a source";
-          if (m_skipGapless) { // Was explicitly set by stateChange interrupt
-              debug() << "...oh, no, just got aborted, skipping EOS";
-              m_skippingEOS = false;
-          }
-      } else {
-          warning() << "aboutToFinishWait timed out!";
-          m_skippingEOS = false;
-      }
+        // Dynamic lock timeout is our friend.
+        // Instead of simply waiting for a fixed amount of ms for the next source, we wait for the
+        // most sensible amount of time. This is whatever amount of time is remaining to play
+        // minus a 0.5 seconds safety delta (time values not precise etc.).
+        // A source for which we have no time or for which the remaining time is < 0.5 seconds is
+        // immediately unlocked again. Otherwise the consumer has as much time as gst gave us to
+        // set a new source.
+        // This in particular prevents pointless excessive locking on sources which have a totalTime
+        // < whatever fixed lock value we have (so that we'd lock longer than what we are playing).
+        // An issue apparent with notification-like sounds, that are rather short and do not need
+        // gapless transitioning. As outlined in https://bugs.kde.org/show_bug.cgi?id=307530
+        unsigned long timeout = 0;
+        if (totalTime() <= 0 || (remainingTime() - 500 <= 0))
+            timeout = 0;
+        else
+            timeout = remainingTime() - 500;
+
+        debug() << "waiting for" << timeout;
+        if (m_aboutToFinishWait.wait(&m_aboutToFinishLock, timeout)) {
+            debug() << "Finally got a source";
+            if (m_skipGapless) { // Was explicitly set by stateChange interrupt
+                debug() << "...oh, no, just got aborted, skipping EOS";
+                m_skippingEOS = false;
+            }
+        } else {
+            warning() << "aboutToFinishWait timed out!";
+            m_skippingEOS = false;
+        }
     } else {
-      debug() << "Skipping gapless audio";
-      m_skippingEOS = false;
+        debug() << "Skipping gapless audio";
+        m_skippingEOS = false;
     }
     m_handlingAboutToFinish = false;
     m_aboutToFinishLock.unlock();
